/* FILE: osivx.h 
 * 
 * Copyright (c) 2002 James A. Strother 
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 * 
 */ 

#ifndef __OSIVX_H__
#define __OSIVX_H__

/* get standard integers */
#include <stdint.h>
#include <stdio.h>

/*
 *  The following definitions are for public use
 *
 */
 
/* define true/false */
#define OSIVX_TRUE ((osivx_int)1) 
#define OSIVX_FALSE ((osivx_int)0) 

/* set string length */
#define OSIVX_STRLEN 32767
#define OSIVX_STRLENS "32767"

/* define input/output types */
typedef uint16_t osivx_iotag_t;
typedef int32_t osivx_ioint_t;
typedef double osivx_ioreal_t;

/* if osivx_ioreal_t is every not
 * the same type as osivx_real_t
 * then OSIVX_SWAP_REAL must be
 * defined as 1
 */
#undef OSIVX_SWAP_REAL


/* osivx_real is used when performing
 * floating point arithmetic, though
 * results are always cast to osivx_ioreal
 * before being exported from library
 */
typedef double osivx_real;


/* The following defines the integer types
 * used by the osivx library.  Since many
 * integer operations run near the limits
 * of the type, special care must be taken
 * to avoid integer overflow.  Here we
 * define types for 8-bit pixels and frames
 * with sizes of at most 16384 x 16384.
 */

/* OSIVX_BIG_WIND is the maximum size of
 * the windom in any dimension.
 */
#define OSIVX_BIG_WIND (255)

/* OSIVX_BIG_FRAME is the maximum size of
 * the image in any dimension.
 */
#define OSIVX_BIG_FRAME (16384)

/* OSIVX_BIG_MOVIE is the maximum length
 * of a movie sequence
 */
#define OSIVX_BIG_MOVIE (16777216)


/* osivx_int must be defined to provide
 * resonable margins on the above limits
 */
typedef int32_t osivx_int;

/* the following types define permutations
 * on the pixel type.  They breakdown as
 * such: osivx_imcp_t holds the value of
 * a pixel, osivx_imsp_t holds the
 * difference of two pixel values, and
 * osivx_immp_t holds the multiplication
 * of two pixel values.  The rest are
 * the sum of the above types over some
 * domain.  Every osivx_im?w_t is the
 * sum of the corresponding osivx_im?w_t
 * over a window. Every osivx_im?f_t is the
 * sum of the corresponding osivx_im?f_t
 * over a frame.
 */
typedef uint8_t osivx_imcp_t;
typedef int16_t osivx_imsp_t;
typedef uint16_t osivx_immp_t;
typedef uint32_t osivx_imcw_t;
typedef int32_t osivx_imsw_t;
typedef uint32_t osivx_immw_t;
typedef uint64_t osivx_imcf_t;
typedef int64_t osivx_imsf_t;
typedef uint64_t osivx_immf_t;


/* define max pixel value */
#define OSIVX_IMCP_MAX (~(osivx_imcp_t)0)

/* define tag values */
#define OSIVX_TAG_ISCLOSE      0x0040   /* 0000 0000 0100 0000 */ 
#define OSIVX_TAG_ISBASE       0x0080   /* 0000 0000 1000 0000 */ 

#define OSIVX_TAG_HEADER       0x0000   /* 0000 0000 0000 0000 */ 
#define OSIVX_TAG_BODY         0x0001   /* 0000 0000 0000 0001 */ 
#define OSIVX_TAG_ARGINT       0x0082   /* 0000 0000 1000 0010 */ 
#define OSIVX_TAG_ARGSTR       0x0083   /* 0000 0000 1000 0011 */  
#define OSIVX_TAG_FRAME        0x0004   /* 0000 0000 0000 0100 */ 
#define OSIVX_TAG_FRAMENUM     0x0085   /* 0000 0000 1000 0101 */  
#define OSIVX_TAG_VECTORS      0x0086   /* 0000 0000 1000 0110 */ 
#define OSIVX_TAG_MAPBOX       0x0007   /* 0000 0000 0000 0111 */ 
#define OSIVX_TAG_MAPNUM       0x0088   /* 0000 0000 1000 1000 */ 
#define OSIVX_TAG_MAP          0x0089   /* 0000 0000 1000 1001 */ 

#define OSIVX_GAT_HEADER       (~(osivx_iotag_t)OSIVX_TAG_HEADER) 
#define OSIVX_GAT_BODY         (~(osivx_iotag_t)OSIVX_TAG_BODY) 
#define OSIVX_GAT_ARGINT       (~(osivx_iotag_t)OSIVX_TAG_ARGINT) 
#define OSIVX_GAT_ARGSTR       (~(osivx_iotag_t)OSIVX_TAG_ARGSTR) 
#define OSIVX_GAT_FRAME        (~(osivx_iotag_t)OSIVX_TAG_FRAME) 
#define OSIVX_GAT_FRAMENUM     (~(osivx_iotag_t)OSIVX_TAG_FRAMENUM) 
#define OSIVX_GAT_VECTORS      (~(osivx_iotag_t)OSIVX_TAG_VECTORS) 
#define OSIVX_GAT_MAPBOX       (~(osivx_iotag_t)OSIVX_TAG_MAPBOX) 
#define OSIVX_GAT_MAPNUM       (~(osivx_iotag_t)OSIVX_TAG_MAPNUM) 
#define OSIVX_GAT_MAP          (~(osivx_iotag_t)OSIVX_TAG_MAP) 
#define OSIVX_GAT_VECBOX       (~(osivx_iotag_t)OSIVX_TAG_VECBOX)
#define OSIVX_GAT_VECNUM       (~(osivx_iotag_t)OSIVX_TAG_VECNUM)
#define OSIVX_GAT_VEC          (~(osivx_iotag_t)OSIVX_TAG_VEC)

/* define argument keys */
#define OSIVX_KEY_MAX 30
#define OSIVX_KEY_ALG ((osivx_ioint_t)0) 
#define OSIVX_KEY_PPR ((osivx_ioint_t)1) 
#define OSIVX_KEY_ITP ((osivx_ioint_t)2) 
#define OSIVX_KEY_SZX ((osivx_ioint_t)3) 
#define OSIVX_KEY_SZY ((osivx_ioint_t)4) 
#define OSIVX_KEY_STX ((osivx_ioint_t)5) 
#define OSIVX_KEY_STY ((osivx_ioint_t)6) 
#define OSIVX_KEY_OFX ((osivx_ioint_t)7) 
#define OSIVX_KEY_OFY ((osivx_ioint_t)8) 
#define OSIVX_KEY_SPX ((osivx_ioint_t)9) 
#define OSIVX_KEY_SPY ((osivx_ioint_t)10) 
#define OSIVX_KEY_NMX ((osivx_ioint_t)11) 
#define OSIVX_KEY_NMY ((osivx_ioint_t)12) 
#define OSIVX_KEY_WSX ((osivx_ioint_t)13) 
#define OSIVX_KEY_WSY ((osivx_ioint_t)14) 
#define OSIVX_KEY_MDX ((osivx_ioint_t)15) 
#define OSIVX_KEY_MDY ((osivx_ioint_t)16) 
#define OSIVX_KEY_WLX ((osivx_ioint_t)17) 
#define OSIVX_KEY_WLY ((osivx_ioint_t)18) 
#define OSIVX_KEY_SFR ((osivx_ioint_t)19) 
#define OSIVX_KEY_EFR ((osivx_ioint_t)20) 
#define OSIVX_KEY_SKP ((osivx_ioint_t)21) 
#define OSIVX_KEY_SET ((osivx_ioint_t)22) 
#define OSIVX_KEY_NIT ((osivx_ioint_t)23) 
#define OSIVX_KEY_WMP ((osivx_ioint_t)24) 
#define OSIVX_KEY_MVA ((osivx_ioint_t)25) 
#define OSIVX_KEY_DTN ((osivx_ioint_t)26) 
#define OSIVX_KEY_NTH ((osivx_ioint_t)27)
#define OSIVX_KEY_MVB ((osivx_ioint_t)28) 
#define OSIVX_KEY_ITX ((osivx_ioint_t)29) 
#define OSIVX_KEY_ITY ((osivx_ioint_t)30) 

#ifdef OSIVX_WITH_KEYLIST
static const char* osiv_keylist[] =  
  {  
    "xcorr_alg",      /* 00 */
    "pproc_alg",      /* 01 */
    "interp_alg",     /* 02 */
    "image_xsize",    /* 03 */
    "image_ysize",    /* 04 */
    "grid_xstart",    /* 05 */
    "grid_ystart",    /* 06 */
    "disp_xoffset",   /* 07 */
    "disp_yoffset",   /* 08 */
    "grid_xspace",    /* 09 */
    "grid_yspace",    /* 10 */
    "grid_xsize",     /* 11 */
    "grid_ysize",     /* 12 */
    "wind_xsize",     /* 13 */
    "wind_ysize",     /* 14 */
    "disp_xmax",      /* 15 */
    "disp_ymax",      /* 16 */
    "wind_xsuper",    /* 17 */
    "wind_ysuper",    /* 18 */
    "movie_start",    /* 19 */
    "movie_finish",   /* 20 */
    "movie_skip",     /* 21 */
    "movie_set",      /* 22 */
    "num_iter",       /* 23 */
    "write_map",      /* 24 */
    "movie_one",      /* 25 */
    "output",         /* 26 */
    "num_thread",     /* 27 */
    "movie_two",      /* 28 */
    "disp_xiter",     /* 29 */
    "disp_yiter"      /* 30 */
  };  
#endif

/* make error codes */ 
enum {
    OSIVX_EOKAY,         OSIVX_EMEM,
    OSIVX_EALG,          OSIVX_EPREPROC, 
    OSIVX_EINTERP,       OSIVX_EIMAGEXSIZE, 
    OSIVX_EIMAGEYSIZE,   OSIVX_EGRIDXSTART,
    OSIVX_EGRIDYSTART,   OSIVX_EGRIDXOFFSET,
    OSIVX_EGRIDYOFFSET,  OSIVX_EGRIDXSPACING,
    OSIVX_EGRIDYSPACING, OSIVX_EGRIDXSIZE,  
    OSIVX_EGRIDYSIZE,    OSIVX_EWINDXSIZE,
    OSIVX_EWINDYSIZE,    OSIVX_EWINDXSUPER,
    OSIVX_EWINDYSUPER,   OSIVX_EDISPXMAX,
    OSIVX_EDISPYMAX,     OSIVX_EMOVIESTART,
    OSIVX_EMOVIEFINISH,  OSIVX_EMOVIESKIP,
    OSIVX_EMOVIESET,     OSIVX_ENUMITER,
    OSIVX_ENUMTHRD,      OSIVX_EMOVAFILE,
    OSIVX_EMOVBFILE,     OSIVX_EVECFILE,
    OSIVX_EWRITEMAP,     OSIVX_EXGRID,
    OSIVX_EYGRID,        OSIVX_EMOVDIR,
    OSIVX_EMOVRAW,       OSIVX_EMOVTIFF,
    OSIVX_EMOVTYPE,      OSIVX_EMOVXSIZE,
    OSIVX_EMOVYSIZE,     OSIVX_EFILEMODE,
    OSIVX_EFILEPOS,      OSIVX_EFILEREAD,
    OSIVX_EFILEWRITE,    OSIVX_EDISPINT,
    OSIVX_EWINDINT,      OSIVX_EFRAMEINT,
    OSIVX_EMOVIEINT,     OSIVX_EIMAGEINT,
    OSIVX_EFFTPLAN,      OSIVX_ETHREAD,
    OSIVX_EMOVXFILE,     OSIVX_EWTHREADS,
    OSIVX_EMOVSETEQ,     OSIVX_EGRID,
    OSIVX_EDISPXITER,    OSIVX_EDISPYITER
};


/* make osivx_err type */
#define OSIVX_ERR_NONE   0
#define OSIVX_ERR_STRING 1
#define OSIVX_ERR_INTEG  2

typedef struct
{
   osivx_ioint_t  type;
   union {
      struct {
         char*       data;
         osivx_ioint_t  len;
      } str;
      osivx_ioint_t     integ;
   } x;
} osivx_err;


/* make osivx_args_t type */ 
typedef struct 
{ 
  osivx_ioint_t alg_xcorr; 
  osivx_ioint_t alg_pproc; 
  osivx_ioint_t alg_interp; 
  osivx_ioint_t image_xsize; 
  osivx_ioint_t image_ysize; 
  osivx_ioint_t grid_xstart; 
  osivx_ioint_t grid_ystart; 
  osivx_ioint_t grid_xspacing; 
  osivx_ioint_t grid_yspacing; 
  osivx_ioint_t grid_xsize; 
  osivx_ioint_t grid_ysize; 
  osivx_ioint_t wind_xsize; 
  osivx_ioint_t wind_ysize; 
  osivx_ioint_t disp_xoffset; 
  osivx_ioint_t disp_yoffset; 
  osivx_ioint_t disp_xmax; 
  osivx_ioint_t disp_ymax;
  osivx_ioint_t disp_xiter;
  osivx_ioint_t disp_yiter;
  osivx_ioint_t wind_xsuper; 
  osivx_ioint_t wind_ysuper; 
  osivx_ioint_t movie_start; 
  osivx_ioint_t movie_finish; 
  osivx_ioint_t movie_skip; 
  osivx_ioint_t movie_set; 
  osivx_ioint_t num_iter; 
  osivx_ioint_t num_thrd; 
  osivx_ioint_t write_map; 
  osivx_err err_info;
  char* movie_aname;
  char* movie_bname;
  char* vectors_name; 
  FILE* porthole_file;
} osivx_args_t; 


/* make osivx_file_t type */
typedef struct 
{ 
  FILE* stream; 

  int mode;
  int pre_len; 
 
  int swap_bytes; 
  int swap_bits; 
 
  int iotag_size; 
  int ioint_size; 
  int ioreal_size; 
 
  int ver_first; 
  int ver_second; 
  int ver_third; 

  osivx_err* err_info;
} osivx_file_t; 


/* alloca.c */
void *osivx_malloc(size_t);
void *osivx_calloc(size_t, size_t);
void osivx_free(void*);

/* error.c */
const char* osivx_strerr(osivx_int);

/* osivx.c */
osivx_int osivx_run(osivx_args_t*);

/* file.c */
osivx_file_t* osivx_fsopen(FILE*, const char*);
osivx_file_t* osivx_fopen(const char*, const char*);
osivx_int osivx_fclose(osivx_file_t*);
osivx_int osivx_ftell(osivx_file_t*, long*);
osivx_int osivx_fseek(osivx_file_t*, long, int);

/* save.c */
osivx_int osivx_fwrite_tag(osivx_file_t*, osivx_iotag_t);
osivx_int osivx_fwrite_argint(osivx_file_t*, osivx_ioint_t, osivx_ioint_t);
osivx_int osivx_fwrite_argstr(osivx_file_t*, osivx_ioint_t, const char*);
osivx_int osivx_fwrite_framenum(osivx_file_t*, osivx_ioint_t);
osivx_int osivx_fwrite_vectors(osivx_file_t*, osivx_ioint_t, osivx_ioreal_t*);
osivx_int osivx_fwrite_mapnum(osivx_file_t*, osivx_ioint_t);
osivx_int osivx_fwrite_map(osivx_file_t*, osivx_ioint_t, osivx_ioreal_t*);
osivx_int osivx_fwrite_header(osivx_file_t*, osivx_args_t*);

/* load.c */
osivx_int osivx_fread_ver(osivx_file_t*, int*);
osivx_int osivx_fread_tag(osivx_file_t*, osivx_iotag_t*);
osivx_int osivx_fread_argint(osivx_file_t*, osivx_ioint_t*, osivx_ioint_t*);
osivx_int osivx_fread_argstr(osivx_file_t*, osivx_ioint_t*, char**);
osivx_int osivx_fread_framenum(osivx_file_t*, osivx_ioint_t*);
osivx_int osivx_fread_vectors(osivx_file_t*, osivx_ioint_t*, osivx_ioreal_t**);
osivx_int osivx_fread_mapnum(osivx_file_t*, osivx_ioint_t*);
osivx_int osivx_fread_map(osivx_file_t*, osivx_ioint_t*, osivx_ioreal_t**);
osivx_int osivx_tagcmp(osivx_iotag_t, osivx_iotag_t);

osivx_int osivx_fscan_totag(osivx_file_t*, osivx_iotag_t);
osivx_int osivx_fscan_toframe(osivx_file_t*, osivx_int);
osivx_int osivx_fscan_tomap(osivx_file_t*, osivx_int);

osivx_int osivx_fgrab_argint(osivx_file_t*, osivx_ioint_t, osivx_ioint_t*);
osivx_int osivx_fgrab_argstr(osivx_file_t*, osivx_ioint_t, char**);
osivx_int osivx_fgrab_vectors(osivx_file_t*, osivx_int, osivx_ioint_t*, osivx_ioreal_t**);
osivx_int osivx_fgrab_map(osivx_file_t*, osivx_int, osivx_int, osivx_ioint_t*, osivx_ioreal_t**);

#endif 
